#
# This file is part of TBX-Min
#
# This software is copyright (c) 2016 by Alan Melby.
#
# This is free software; you can redistribute it and/or modify it under
# the same terms as the Perl 5 programming language system itself.
#

# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v5.043.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "Module::Build" => "0.28",
    "perl" => "5.006"
  },
  "dist_abstract" => "Read, write and edit TBX-Min files",
  "dist_author" => [
    "Nathan Glenn <garfieldnate\@gmail.com>, James Hayes <james.s.hayes\@gmail.com>"
  ],
  "dist_name" => "TBX-Min",
  "dist_version" => "0.08",
  "license" => "perl",
  "module_name" => "TBX::Min",
  "recursive_test_files" => 1,
  "requires" => {
    "Carp" => 0,
    "Class::Tiny" => 0,
    "DateTime::Format::ISO8601" => 0,
    "Import::Into" => 0,
    "Path::Tiny" => 0,
    "Try::Tiny" => 0,
    "XML::Twig" => 0,
    "autodie" => 0,
    "perl" => "5.021008",
    "strict" => 0,
    "subs" => 0,
    "warnings" => 0
  },
  "test_requires" => {
    "File::Spec" => 0,
    "File::Temp" => 0,
    "FindBin" => 0,
    "IO::Handle" => 0,
    "IPC::Open3" => 0,
    "Test::Deep" => 0,
    "Test::Exception" => 0,
    "Test::More" => "0.88",
    "Test::NoWarnings" => 0,
    "Test::XML" => 0,
    "blib" => "1.01",
    "perl" => "5.021008"
  }
);


my %fallback_build_requires = (
  "File::Spec" => 0,
  "File::Temp" => 0,
  "FindBin" => 0,
  "IO::Handle" => 0,
  "IPC::Open3" => 0,
  "Module::Build" => "0.28",
  "Test::Deep" => 0,
  "Test::Exception" => 0,
  "Test::More" => "0.88",
  "Test::NoWarnings" => 0,
  "Test::XML" => 0,
  "blib" => "1.01",
  "perl" => "5.021008"
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
